@@ -689,7 +689,7 @@ plot.msm <- function(x, from=NULL, to=NULL, range=NULL, covariates="mean", legen
     }
     if (is.null(to)){
         if (length(absorbing.msm(x))==0)
-            stop("\"to\" not specified, and no absorbing state in the model")
+            stop("\"to\" not specified, and no absorbing state. See help(plot.msm)")
         to <- max(absorbing.msm(x))
     }
     else {
@@ -1843,47 +1843,81 @@ odds.msm <- function(x, odds.scale = 1, cl = 0.95)
     odds.list
 }
 
-viterbi.msm <- function(x, normboot=FALSE)
+viterbi.msm <- function(x, normboot=FALSE, newdata=NULL)
 {
-    if (!inherits(x, "msm")) stop("expected x to be a msm model")
-    if (x$cmodel$ncens > 0 && !x$hmodel$hidden) {
-        ## If censoring but not HMM, then define an identity HMM with
-        ## true state known at every time except censoring times
-        hmod <- vector(x$qmodel$nstates, mode="list")
-        for (i in 1:x$qmodel$nstates)
-            hmod[[i]] <- hmmIdent(i)
-        x$hmodel <- msm.form.hmodel(hmod, est.initprobs=FALSE)
-        x$hmodel <- c(x$hmodel, list(ncovs=rep(rep(0,x$hmodel$nstates),x$hmodel$npars), ncoveffs=0, nicovs=rep(0,x$hmodel$nstates-1), nicoveffs=0))
-        x$data$mf$"(obstrue)" <- ifelse(x$data$mf$"(state)" %in% x$cmodel$censor, 0, (x$data$mf$"(state)"))
-        x$data$mm.hcov <- vector(mode="list", length=x$hmodel$nstates) # reqd by msm.add.hmmcovs
-        for (i in seq_len(x$hmodel$nstates))
-            x$data$mm.hcov[[i]] <- model.matrix(~1, x$data$mf)
-        x$paramdata$allinits <- c(x$paramdata$allinits,x$hmodel$pars)
-        x$paramdata$constr <- c(x$paramdata$constr,max(x$paramdata$constr)+seq_along(x$hmodel$pars))
-    }
-    if (x$hmodel$hidden) {        
-        params <-
-          if (normboot)
-              rmvnorm(1, x$paramdata$opt$par, x$covmat[x$paramdata$optpars,x$paramdata$optpars])
-          else x$paramdata$opt$par
-        ret <- Ccall.msm(params, do.what="viterbi", expand.data(x),
-                               x$qmodel, x$qcmodel, x$cmodel, x$hmodel, x$paramdata)
-        fitted <- ret[[1]]; pstate <- ret[[2]]
-        fitted <- fitted + 1
-    }
-    else { 
-        fitted <- x$data$mf$"(state)"
-        pstate <- NULL
-    }
-    if (!is.null(x$qmodel$phase.states)){
-        fitted <- x$qmodel$phase.labs[fitted]
-    }
-    ret <- data.frame(subject = x$data$mf$"(subject)",
-               time = x$data$mf$"(time)",
-               observed = x$data$mf$"(state)",
-               fitted = fitted)
-    if (!is.null(pstate)) ret$pstate <- pstate
-    ret
+  if (!inherits(x, "msm")) stop("expected x to be a msm model")
+  
+  if (!is.null(newdata)){
+    ## initialise msm with new data but do not fit (hence fixedpars = TRUE)
+    newcall <- x$call
+    newcall$data <- substitute(newdata)
+    newcall$fixedpars <- TRUE
+    xnew <- try(eval(newcall))
+  } else {
+    xnew <- x
+  }
+  xdata <- expand.data(xnew)
+    
+  if (x$cmodel$ncens > 0 && !x$hmodel$hidden) {
+    ## If censoring but not HMM, then define an identity HMM with
+    ## true state known at every time except censoring times
+    hmod <- vector(x$qmodel$nstates, mode="list")
+    for (i in 1:x$qmodel$nstates)
+      hmod[[i]] <- hmmIdent(i)
+    x$hmodel <- msm.form.hmodel(hmod, est.initprobs=FALSE)
+    x$hmodel <- c(x$hmodel, list(ncovs=rep(rep(0,x$hmodel$nstates),x$hmodel$npars), 
+                                 ncoveffs=0, nicovs=rep(0,x$hmodel$nstates-1), nicoveffs=0))
+    xdata$mf$"(obstrue)" <- ifelse(xdata$mf$"(state)" %in% x$cmodel$censor, 
+                                          0, (xdata$mf$"(state)"))
+    xdata$mm.hcov <- vector(mode="list", length=x$hmodel$nstates) # reqd by msm.add.hmmcovs
+    for (i in seq_len(x$hmodel$nstates))
+      xdata$mm.hcov[[i]] <- model.matrix(~1, xdata$mf)
+    x$paramdata$allinits <- c(x$paramdata$allinits,x$hmodel$pars)
+    x$paramdata$constr <- c(x$paramdata$constr,max(x$paramdata$constr)+seq_along(x$hmodel$pars))
+      npts <- attr(xdata$mf, "npts")
+      initstate <- xdata$mf$"(state)"[!duplicated(xdata$mf$"(subject)")]
+      initp <- matrix(0,nrow=npts,ncol=x$hmodel$nstates)
+      for (i in 1:npts){
+          if (initstate[i] %in% x$cmodel$censor) {
+              cs <- x$cmodel$states_list[[as.character(initstate[i])]]
+              initp[i,cs] <- 1/length(cs)
+          }
+          else initp[i,initstate[i]] <- 1
+      }
+      x$hmodel$initprobs <- initp
+  }
+
+    if (x$hmodel$hidden) {
+    if (normboot)
+      params <- rmvnorm(1, x$paramdata$opt$par, x$covmat[x$paramdata$optpars,x$paramdata$optpars])
+    else
+      params <- x$paramdata$opt$par
+    
+    ret <- Ccall.msm(params,
+                     do.what="viterbi",
+                     xdata,
+                     x$qmodel, x$qcmodel, x$cmodel, x$hmodel, x$paramdata
+    )
+    fitted <- ret[[1]]
+    pstate <- ret[[2]]
+    fitted <- fitted + 1
+  } else {
+    fitted <- xdata$mf$"(state)"
+    pstate <- NULL
+  }
+  
+  if (!is.null(x$qmodel$phase.states)){
+    fitted <- x$qmodel$phase.labs[fitted]
+  }
+  ret <- data.frame(
+    subject = xdata$mf$"(subject)",
+    time = xdata$mf$"(time)",
+    observed = xdata$mf$"(state)",
+    fitted = fitted
+  )
+  if (!is.null(pstate))
+    ret$pstate <- pstate
+  ret
 }
 
 scoreresid.msm <- function(x, plot=FALSE){
